﻿using ProjectMotorboat.Exceptions;

namespace ProjectMotorboat.CollectionGenericObjects;
public class MassiveGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{

    private T?[] _collection;

    public int Count => _collection.Length;

    public int MaxCount
    {
        get
        {
            return _collection.Length;
        }
        set
        {
            if (value > 0)
            {
                if (Count > 0)
                {
                    Array.Resize(ref _collection, value);
                }
                else
                {
                    _collection = new T?[value];
                }
            }
        }
    }

    public CollectionType GetCollectionType => CollectionType.Massive;

    public MassiveGenericObjects()
    {
        _collection = Array.Empty<T?>();
    }

    public T? Get(int position)
    {
        if (position < 0 || position >= Count) throw new PositionOutOfCollectionException(position);
        if (_collection[position] == null) throw new ObjectNotFoundException(position);
        return _collection[position];
    }
    public int Insert(T obj, IEqualityComparer<T?>? comparer = null)
    {
        return Insert(obj, 0, comparer);
    }

    public int Insert(T obj, int position, IEqualityComparer<T?>? comparer = null)
    {
        if (position < 0 || position >= Count)
        {
            throw new PositionOutOfCollectionException(position);
        }
        if (comparer != null)
        {
            if (_collection.Contains(obj, comparer))
            {
                throw new CollectionOverflowException();
            }
        }
        if (_collection[position] == null)
        {
            _collection[position] = obj;
            return position;
        }
        for (int i = position + 1; i < Count; i++)
        {
            if (_collection[i] == null)
            {
                _collection[i] = obj;
                return i;
            }
        }
        for (int i = position - 1; i >= 0; i--)
        {
            if (_collection[i] == null)
            {
                _collection[i] = obj;
                return i;
            }
        }
        throw new CollectionOverflowException(Count);
    }

    public T Remove(int position)
    {
        if (position < 0 || position >= Count) throw new PositionOutOfCollectionException(position);

        if (_collection[position] == null) throw new ObjectNotFoundException(position);

        T? temp = _collection[position];
        _collection[position] = null;
        return temp;
    }

    public IEnumerable<T> GetItems()
    {
        for (int i = 0; i < _collection.Length; ++i)
        {
            yield return _collection[i];
        }
    }
    public void CollectionSort(IComparer<T?> comparer)
    {
        Array.Sort(_collection, comparer);
    }
}




